#include "Notification.h"
/**
   AbstractCommand is the base class for all commands.
   It provides do/undo operations as well as getDescription and
   getState operations for the easy tracking of the excecuted commands.
   (quite useful when keeping a menu of last performed operations).

 */

class AbstractCommand 
{
 public:
  /**@name Comparision operators
   * The comparision operators in the base AbstractCommand are useless.
   * Many other Command children will keep this behavior void. Nevertheless,
   * for extensibility reasons, they are still provided.
   */
  //@{
  
  /// equality operator
  virtual int operator==();

  /// unequality operator
  virtual int operator!=();

  /// increment operator
  virtual void  operator++();
  //@}

  /**@name Do / Undo methods
   */
  //@{
  
  /// Pure virtual operation that in child classes encapsulates the logic of the change
  virtual Notification do() = 0;

  /// Pure virtual operation that in child classes encapsulates the logic of undoing a change
  virtual Notification undo() = 0;

  /**Pure virtual operation that in child classes returns the description of the operation
   * ( particulary useful for undo/redo lists presented to the user)
   */
  virtual string getDescription() = 0;
  //@}
};


