//---------------------------------------------------------------------------
//Header Includes
//---------------------------------------------------------------------------
#include "AbstractCommand.h"
#include "xmlparse.h"

/**
   AddElementCommand encapsulates the additon of an element to the document.
   Add element command is constructed using the current node and upon excecution 
   it insertes its product there, even if another view has changed the current node
   to another value.

 */

class AddElementCommand : AbstractCommand 
{
 public:
  /**@name Overriden Do / Undo methods
   */
  //@{
  
  /** The call to do() will generate the insertion of product into parent (the current node
   * at the time of the creation of the object).
   * @return a deep Notification object saying that parent has changed.
   * @see Notification.h
   */
  Notification do();

  /** The call to undo() will generate the removal of product as child of parent (the current node
   * at the time of the creation of the object).
   * @return a deep Notification object saying that parent has changed.
   * @see Notification.h
   */
  Notification void undo();
  //@}

  /** @name Get methods
   *
   */
  //@{
  /** Returns the description of the operation
   * (Actually the string "Add Element %s" where %s is the name of the element is returned)
   */
  string getDescription();

  /// gets the parent
  xmlNode *getParent();

  /// gets the product
  xmlNode *getProduct();
  //@}

  /** @name Set methods
   *
   */
  //@{
  /// sets the product. Note that there is no set parent, since is is a const set upon creation
  void setProduct(xmlNode *newValue);
  //@}

 private:

  /// the current node at the time of the creation of the object
  const xmlNode *parent;

  /// the node to be inserted as child of parent 
  xmlNode *product;
};
