//---------------------------------------------------------------------------
//Header Includes
//---------------------------------------------------------------------------
#ifndef CommandManager_H
#define CommandManager_H
#include <list>
#include "AbstractCommand.h"

//---------------------------------------------------------------------------
//Constants
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Class definition
//---------------------------------------------------------------------------
/**
   CommandManager is in charge of managing the list of commands
   that arrive from the views. Its mission is to sequentially ask each command 
   to do/undo(). Normally, all the CommandManager does is to execute the incoming
   command, however, it can, at the controller's request, redo or undo an arbitrary 
   number of commands (limited only by the capacity attribute).
   
 */

class CommandManager {
 public:
  /**@name Get Methods
   */
  //@{
  /// Get the allowed number of undos (size of the command buffer)
  unsigned int inline getSize();
  //@}


  /**@name Set Methods
   */
  //@{
  /// Get the allowed number of undos (size of the command buffer)
  void inline setSize(unsigned int newSize);
  //@}

  /**@name Command Management methods
   */
  //@{
  /** 
      queueCommand is called whenever there is a new command issued by the views.
      If there is space for the new command, it is added. If there is no space, a
      SpaceExceeded exception is thrown.
      @param c The command to be added to the end of the queue.
      @exception SpaceExceeded if the queue size is exceeded
   */
  inline void queueCommand(AbstractCommand *c) raises SpaceExceeded;

  /** 
      Basically,call the do() method of the current command and get prepared to
      work on the next ones. Note that this must not be called explicitly unless
      a redo is due: The command is normally executed automatically.
      @return the notification issued by the do method  on the command.
   */
  inline Notification excecuteCurrent();

  /** 
      Basically,call the undo() method of the current command and get prepared to
      work on the next ones. Note that this must not be called explicitly unless
      an undo is due.
      @return the not Notification issued by the do method  on the command.
   */
  inline Notification unExcecuteCurrent();
  //@}

 protected:
 private:
  /// amount of commands held in memory
  unsigned int size;
  /// list of commands
  list<AbstractCommand *> commands;
  /// current commands
  list<AbstractCommand *>::iterator current;
};
#endif













